@@ -84,6 +84,7 @@ class Imgur():
         self._client_secret = client_secret
         self._access_token = None
         self._refresh_token = None
+        self._request = self._connect.request
 
         self._auth_url = (
             'https://api.imgur.com/oauth2/authorize?'
@@ -96,23 +97,14 @@ class Imgur():
         self._enter_token_msg = 'Enter PIN code displayed in the browser: '
         self._no_album_msg = 'Do not move to any album'
 
-    def retry(tries=2, delay=1, errors=(ImgurError, httplib.BadStatusLine)):
+    def retry(errors=(ImgurError, httplib.BadStatusLine)):
         """Retry calling the decorated function using an exponential backoff.
 
         http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
         original from: http://wiki.python.org/moin/PythonDecoratorLibrary#Retry
-
-        :param tries: number of times to try (not retry) before giving up
-        :type tries: int
-        :param delay: initial delay between retries in seconds
-        :type delay: int
         """
-
-        if tries < 0:
-            raise ValueError("tries must be 0 or greater")
-
-        if delay <= 0:
-            raise ValueError("delay must be greater than 0")
+        tries = 2
+        delay = 1
 
         def deco_retry(f):
             def f_retry(self, *args, **kwargs):
@@ -141,7 +133,6 @@ class Imgur():
         :return: A list include dialog command, ex: ['kdialog', '--msgbox', 'hello']
         :rtype: list
         """
-        pass
 
     def show_error_and_exit(self, msg='Error'):
         """Display error message and exit the program
@@ -165,33 +156,20 @@ class Imgur():
         Set tokens to None if can't be found in config
         """
         parser = SafeConfigParser()
-        parser.read(self.CONFIG_PATH)
+        fp = open(self.CONFIG_PATH)
+        parser.readfp(fp)
 
         try:
             self._access_token = parser.get('Token', 'access_token')
         except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
             logger.warning('Can\'t find access token, set to empty')
             self._access_token = None
-
         try:
             self._refresh_token = parser.get('Token', 'refresh_token')
         except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
             logger.warning('Can\'t find refresh token, set to empty')
             self._refresh_token = None
-
-    def _request(self, method, url, body, headers):
-        """Private API for request
-
-        :param method: `GET` or `POST`
-        :type method: str
-        :param url: url
-        :type url: str
-        :param body: Body part of the request
-        :type body: str or None
-        :param headers: The header part of the request
-        :type headers: dict
-        """
-        self._connect.request(method, url, body, headers)
+        fp.close()
 
     def _get_json_response(self):
         """Get the json response of request
@@ -289,7 +267,6 @@ class Imgur():
         :return: A list include dialog command
         :rtype: list
         """
-        pass
 
     @abstractmethod
     def get_enter_pin_dialog_args(self, token_msg):
@@ -300,7 +277,6 @@ class Imgur():
         :return: A list include dialog command
         :rtype: list
         """
-        pass
 
     def ask_pin(self, auth_msg, auth_url, enter_token_msg):
         """Ask user for pin code
@@ -355,7 +331,8 @@ class Imgur():
                 {
                     'client_id': self._client_id,
                     'client_secret': self._client_secret,
-                    'grant_type': 'pin', 'pin': pin
+                    'grant_type': 'pin',
+                    'pin': pin
                 }
             ),
             headers
@@ -383,8 +360,6 @@ class Imgur():
 
     def write_tokens_to_config(self):
         """Write token value to the config
-        There will be maybe more setting needed to be written to config
-        So I just pass `result`
         """
         logger.debug('Access token: %s', self._access_token)
         logger.debug('Refresh token: %s', self._refresh_token)
@@ -405,7 +380,6 @@ class Imgur():
         :return: A list include dialog command, ex: ['kdialog', '--msgbox', 'hi']
         :rtype: list
         """
-        pass
 
     def ask_image_path(self):
         """Display a file dialog and prompt the user to select a image
@@ -447,7 +421,6 @@ class Imgur():
         :return: A list include dialog command
         :rtype: list
         """
-        pass
 
     def ask_album_id(self, albums):
         """Ask user to choose a album to upload or not belong to any album
@@ -481,7 +454,6 @@ class Imgur():
         :return: A list include dialog command
         :rtype: list
         """
-        pass
 
     def show_link(self, image_link, delete_hash):
         """Show image link
@@ -515,16 +487,20 @@ class Imgur():
             return mimetypes.guess_type(filename)[0] or 'application/octet-stream'
 
         def encode_field(field_name):
-            return ('--' + boundary,
-                    'Content-Disposition: form-data; name="%s"' % field_name,
-                    '', str(data[field_name]))
+            return (
+                '--' + boundary,
+                'Content-Disposition: form-data; name="%s"' % field_name,
+                '', str(data[field_name])
+            )
 
         def encode_file(field_name):
             filename = files[field_name]
-            return ('--' + boundary,
-                    'Content-Disposition: form-data; name="%s"; filename="%s"' % (field_name, filename),
-                    'Content-Type: %s' % get_content_type(filename),
-                    '', open(filename, 'rb').read())
+            return (
+                '--' + boundary,
+                'Content-Disposition: form-data; name="%s"; filename="%s"' % (field_name, filename),
+                'Content-Type: %s' % get_content_type(filename),
+                '', open(filename, 'rb').read()
+            )
 
         boundary = random_string(30)
         lines = []
@@ -535,8 +511,10 @@ class Imgur():
         lines.extend(('--%s--' % boundary, ''))
         body = '\r\n'.join(lines)
 
-        headers = {'content-type': 'multipart/form-data; boundary=' + boundary,
-                   'content-length': str(len(body))}
+        headers = {
+            'content-type': 'multipart/form-data; boundary=' + boundary,
+            'content-length': str(len(body))
+        }
 
         return body, headers
 
@@ -612,10 +590,10 @@ class CLIImgur(Imgur):
         return None
 
     def get_auth_msg_dialog_args(self, auth_msg, auth_url):
-        pass
+        raise NotImplementedError()
 
     def get_enter_pin_dialog_args(self, token_msg):
-        pass
+        raise NotImplementedError()
 
     def ask_pin(self, auth_msg, auth_url, enter_token_msg):
         print(auth_msg + auth_url)
@@ -623,14 +601,14 @@ class CLIImgur(Imgur):
         return pin
 
     def get_ask_image_path_dialog_args(self):
-        pass
+        raise NotImplementedError()
 
     def ask_image_path(self):
         image_path = raw_input('Enter your image location: ')
         return image_path
 
     def get_ask_album_id_dialog_args(self, albums, no_album_msg):
-        pass
+        raise NotImplementedError()
 
     def ask_album_id(self, albums):
         i = 1
@@ -643,11 +621,11 @@ class CLIImgur(Imgur):
         print('{i}) {msg}'.format(i=i, msg=self._no_album_msg))
         data_map.append({'id': None})
         album_number = int(input())
-        # Return album id, number select start from 1, so minus 1
+        # Get [album_number-1] from the list
         return self._get_album_id(data_map, album_number)
 
     def get_show_link_dialog_args(self, links):
-        pass
+        raise NotImplementedError()
 
     def show_link(self, image_link, delete_hash):
         print('Link: {link}'.format(link=image_link.replace('\\', '')))
@@ -698,7 +676,7 @@ class KDEImgur(Imgur):
             args.append('{album[title]}({album[privacy]})'.format(album=album))
             i += 1
         args.append(str(i))
-        args.append(no_album_msg)
+        args.append(no_album_msg + '(public)')
 
         return args
 
@@ -786,9 +764,12 @@ class MacImgur(Imgur):
             stderr=subprocess.PIPE
         )
         album_number = choose_album_dialog.communicate()[0].strip()
-        album_number = album_number[:album_number.find(' ')]
-        if album_number == '':
-            self.show_error_and_exit('n should not be empty')
+        album_number_end = album_number.find(' ')
+        if album_number_end == -1:
+            self.show_error_and_exit(
+                'Return string format error: {{s}}'.format(s=album_number)
+            )
+        album_number = album_number[:album_number_end]
         album_number = int(album_number)
         return self._get_album_id(data_map, album_number)
 
@@ -814,7 +795,6 @@ class MacImgur(Imgur):
         )
         response = show_link_dialog.communicate()[0].strip()
         response = response[response.find(':') + 1:response.find(',')]
-        print(response)
         if response == 'Show delete link':
             delete_link = 'http://imgur.com/delete/{delete}'.format(delete=delete_hash)
             args2 = [
@@ -869,7 +849,7 @@ class ZenityImgur(Imgur):
 
     def get_ask_album_id_dialog_args(self, albums, no_album_msg):
         i = 1
-        arg = [
+        args = [
             'zenity',
             '--list',
             '--text="Choose the album"',
@@ -878,13 +858,14 @@ class ZenityImgur(Imgur):
             '--column=Privacy',
         ]
         for album in albums:
-            arg.append(str(i))
-            arg.append('{album[title]}'.format(album=album))
-            arg.append('{album[privacy]}'.format(album=album))
+            args.append(str(i))
+            args.append('{album[title]}'.format(album=album))
+            args.append('{album[privacy]}'.format(album=album))
             i += 1
-        arg.append(str(i))
-        arg.append(no_album_msg)
-        arg.append('public')
+        args.append(str(i))
+        args.append(no_album_msg)
+        args.append('public')
+        return args
 
     def get_show_link_dialog_args(self, links):
         args = [
